%global forgeurl https://github.com/fastfloat/%{name}/

# Tests are stored in a separate git repository. We should always use the
# latest commit.
%global stf_commit 97a0b2e638feb479387554cf253e346500541e7e
%global stf_forgeurl https://github.com/fastfloat/supplemental_test_files/

Name:           fast_float
Summary:        Fast & exact implementation of C++ from_chars for float/double
Version:        1.1.2
%forgemeta
Release:        1%{?dist}

URL:            %{forgeurl}
# README.md:
#   Licensed under either of Apache License, Version 2.0 or MIT license at your
#   option.
License:        ASL 2.0 or MIT

Source0:        %{forgesource}
Source1:        %{stf_forgeurl}/archive/%{stf_commit}/supplemental_test_files-%{stf_commit}.tar.gz

BuildRequires:  gcc-c++
BuildRequires:  cmake
# Our choice; the default make backend should work just as well
BuildRequires:  ninja-build

BuildRequires:  doctest-devel
# TODO: https://src.fedoraproject.org/rpms/doctest/pull-request/2 was accepted,
# but there has not been a successful build, so doctest still lacks the
# necessary arch-less virtual Provides. Uncomment this (with a Fedora version
# conditional if necessary) when we can.
#
# Doctest is header-only, so we must BR:
#BuildRequires:  doctest-static

BuildRequires:  symlinks

# No compiled binaries are installed, so this would be empty.
%global debug_package %{nil}

# Required for EPEL8:
%undefine __cmake_in_source_build

%global common_description %{expand:
The fast_float library provides fast header-only implementations for the C++
from_chars functions for float and double types. These functions convert ASCII
strings representing decimal values (e.g., 1.3e10) into binary types. We
provide exact rounding (including round to even). In our experience, these
fast_float functions many times faster than comparable number-parsing functions
from existing C++ standard libraries.}

%description %{common_description}


%package devel
Summary:        %{summary}

# Header-only library
Provides:       %{name}-static = %{version}-%{release}

Requires:       cmake-filesystem

%description devel %{common_description}


%prep
%forgeautosetup -p1
%setup -q -T -D -b 1
# Compiling with -Werror makes sense for upstream CI, but is excessively strict
# for downstream builds across a variety of compiler versions.
sed -r -i 's/-Werror//' tests/CMakeLists.txt


%build
# Partially emulate the activity of FetchContent_Declare() in
# tests/CMakeLists.txt so that we can run the tests offline.
mkdir -p '%{_vpath_builddir}/_deps'
ln -s "${PWD}/../supplemental_test_files-%{stf_commit}/" \
    '%{_vpath_builddir}/_deps/supplemental_test_files-src'

%cmake \
    -GNinja \
    -DFETCHCONTENT_FULLY_DISCONNECTED:BOOL=ON \
    -DSYSTEM_DOCTEST:BOOL=ON \
    -DFASTFLOAT_TEST:BOOL=ON
%cmake_build


%install
%cmake_install

# We add symbolic links to the .cmake files installed in the location preferred
# by upstream, from a location that is more conventional for Linux (and is in
# the default CMake module search path).
install -d '%{buildroot}%{_libdir}/cmake/%{name}'
ln -s -v %{buildroot}%{_datadir}/FastFloat/*.cmake \
    '%{buildroot}%{_libdir}/cmake/%{name}/'
# Convert the absolute symlinks in the buildroot to relative ones.
symlinks -c -o '%{buildroot}%{_libdir}/cmake/%{name}/'


%check
%ctest


%files devel
%license LICENSE-APACHE
%license LICENSE-MIT
%doc AUTHORS
%doc CONTRIBUTORS
%doc README.md

%{_includedir}/%{name}

%{_datadir}/FastFloat
%{_libdir}/cmake/%{name}


%changelog
* Mon Jun 21 2021 Benjamin A. Beasley <code@musicinmybrain.net> - 1.1.2-1
- Update to 1.1.2
- Drop fast_float-1.1.1-system-doctest.patch; now merged upstream
- Add EPEL8 support
- Do not compile tests with -Werror

* Mon Jun 21 2021 Benjamin A. Beasley <code@musicinmybrain.net> - 1.1.1-1
- Initial package
